{
    "columns": [
        {
            "aid": "79",
            "attribute": "*"
        }
    ],
    "where": [
        {
            "condition": {
                "aid": "79",
                "attribute": "MeasurementBegin",
                "operator": "OP_IS_NULL"
            }
        }
    ],
    "rowLimit": "5"
}