{
    "columns": [
        {
            "aid": "79",
            "attribute": "Name"
        },
        {
            "aid": "80",
            "attribute": "Name"
        },
        {
            "aid": "54",
            "attribute": "Name"
        },
        {
            "aid": "81",
            "attribute": "Name"
        }
    ],
    "joins": [
        {
            "aidFrom": "79",
            "aidTo": "80",
            "relation": "MeaQuantities",
            "joinType": "JT_OUTER"
        },
        {
            "aidFrom": "80",
            "aidTo": "54",
            "relation": "Unit",
            "joinType": "JT_OUTER"
        },
        {
            "aidFrom": "79",
            "aidTo": "81",
            "relation": "SubMatrices",
            "joinType": "JT_OUTER"
        }
    ],
    "rowLimit": "10"
}